@@ -41,8 +41,8 @@
     (trace-function-foreground . helm-completing-read-symbols)
     (trace-function-background . helm-completing-read-symbols)
     (find-tag . helm-completing-read-default-find-tag)
-    (org-capture . helm-org-completing-read-tags)
-    (org-set-tags . helm-org-completing-read-tags)
+    (xref-find-definitions . helm-completing-read-default-find-tag)
+    (xref-find-references . helm-completing-read-default-find-tag)
     (ffap-alternate-file . nil)
     (tmm-menubar . nil)
     (find-file . nil)
@@ -212,12 +212,17 @@ know what you are doing."
   :group 'helm-mode
   :type 'boolean)
 
+(defface helm-mode-prefix
+    '((t (:background "red" :foreground "black")))
+  "Face used for prefix completion."
+  :group 'helm-mode)
 
 (defvar helm-comp-read-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map helm-map)
     (define-key map (kbd "<C-return>") 'helm-cr-empty-string)
     (define-key map (kbd "M-RET")      'helm-cr-empty-string)
+    (define-key map (kbd "DEL")        'helm-mode-delete-char-backward-maybe)
     map)
   "Keymap for `helm-comp-read'.")
 
@@ -434,7 +439,8 @@ If COLLECTION is an `obarray', a TEST should be needed. See `obarray'."
                             (alistp t)
                             (candidate-number-limit helm-candidate-number-limit)
                             multiline
-                            allow-nest)
+                            allow-nest
+                            (group 'helm))
   "Read a string in the minibuffer, with helm completion.
 
 It is helm `completing-read' equivalent.
@@ -515,7 +521,7 @@ Keys description:
 
 - NOMARK: When non--nil don't allow marking candidates.
 
-- ALISTP: \(default is non--nil\) See `helm-comp-read-get-candidates'.
+- ALISTP: (default is non--nil) See `helm-comp-read-get-candidates'.
 
 - CANDIDATES-IN-BUFFER: when non--nil use a source build with
   `helm-source-in-buffer' which is much faster.
@@ -529,6 +535,8 @@ Keys description:
 
 - MULTILINE: See multiline in `helm-source'.
 
+- GROUP: See group in `helm-source'.
+
 Any prefix args passed during `helm-comp-read' invocation will be recorded
 in `helm-current-prefix-arg', otherwise if prefix args were given before
 `helm-comp-read' invocation, the value of `current-prefix-arg' will be used.
@@ -596,6 +604,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                        :persistent-action persistent-action
                        :persistent-help persistent-help
                        :keymap loc-map
+                       :group group
                        :mode-line mode-line
                        :help-message help-message
                        :action action-fn))
@@ -612,6 +621,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                   :persistent-help persistent-help
                   :fuzzy-match fuzzy
                   :keymap loc-map
+                  :group group
                   :mode-line mode-line
                   :help-message help-message
                   :action action-fn
@@ -628,6 +638,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                     :persistent-action persistent-action
                     :fuzzy-match fuzzy
                     :keymap loc-map
+                    :group group
                     :persistent-help persistent-help
                     :mode-line mode-line
                     :help-message help-message
@@ -927,6 +938,7 @@ See documentation of `completing-read' and `all-completions' for details."
        (initial-input default-directory)
        (buffer "*Helm file completions*")
        test
+       noret
        (case-fold helm-file-name-case-fold-search)
        preselect
        history
@@ -955,6 +967,9 @@ Keys description:
 
 - TEST: A predicate called with one arg 'candidate'.
 
+- NORET: Allow disabling helm-ff-RET (have no effect if helm-ff-RET
+                                      isn't bound to RET).
+
 - CASE-FOLD: Same as `helm-case-fold-search'.
 
 - PRESELECT: helm preselection.
@@ -1004,11 +1019,13 @@ Keys description:
          (hist (and history (helm-comp-read-get-candidates
                              history nil nil alistp)))
          (minibuffer-completion-confirm must-match)
+         (helm-ff--RET-disabled noret)
          (must-match-map (when must-match
                            (let ((map (make-sparse-keymap)))
                              (define-key map (kbd "RET")
                                (let ((fn (lookup-key helm-read-file-map (kbd "RET"))))
-                                 (if (eq fn 'helm-ff-RET)
+                                 (if (and (eq fn 'helm-ff-RET)
+                                          (null helm-ff--RET-disabled))
                                      #'helm-ff-RET-must-match
                                    #'helm-confirm-and-exit-minibuffer)))
                              map)))
@@ -1136,6 +1153,7 @@ Don't use it directly, use instead `helm-read-file-name' in your programs."
          ;; Append the two extra args needed to set the buffer and source name
          ;; in helm specialized functions.
          (any-args (append def-args (list str-command buf-name)))
+         (reading-directory (eq predicate 'file-directory-p))
          helm-completion-mode-start-message ; Be quiet
          helm-completion-mode-quit-message  ; Same here
          fname)
@@ -1199,14 +1217,15 @@ Don't use it directly, use instead `helm-read-file-name' in your programs."
                        :initial-input (expand-file-name init dir)
                        :alistp nil
                        :must-match mustmatch
-                       :test predicate))))
+                       :test predicate
+                       :noret reading-directory))))
       (and ido-mode (ido-mode -1))
       (helm-mode 1)
       ;; Same comment as in `helm--completing-read-default'.
       (setq this-command current-command))
     (if (and
          ;; Using `read-directory-name'.
-         (eq predicate 'file-directory-p)
+         reading-directory
          ;; `file-name-as-directory' return "./" when FNAME is
          ;; empty string.
          (not (string= fname "")))
@@ -1250,6 +1269,40 @@ The `helm-find-files' history `helm-ff-history' is used here."
   "Default sort function for completion-in-region."
   (sort candidates 'helm-generic-sort-fn))
 
+(defun helm-mode--completion-in-region-initial-input (str)
+  (propertize str 'read-only t 'face 'helm-mode-prefix 'rear-nonsticky t))
+
+(defun helm-mode-delete-char-backward-1 ()
+  (interactive)
+  (condition-case err
+      (call-interactively 'delete-backward-char)
+    (text-read-only
+     (if (with-selected-window (minibuffer-window)
+           (not (string= (minibuffer-contents) "")))
+         (message "Trying to delete prefix completion, next hit will quit")
+       (user-error "%s" (car err))))))
+(put 'helm-mode-delete-char-backward-1 'helm-only t)
+
+(defun helm-mode-delete-char-backward-2 ()
+  (interactive)
+  (condition-case _err
+      (call-interactively 'delete-backward-char)
+    (text-read-only
+     (unless (with-selected-window (minibuffer-window)
+               (string= (minibuffer-contents) ""))
+       (with-helm-current-buffer
+         (run-with-timer 0.1 nil (lambda ()
+                                   (call-interactively 'delete-backward-char))))
+       (helm-keyboard-quit)))))
+(put 'helm-mode-delete-char-backward-2 'helm-only t)
+
+(helm-multi-key-defun helm-mode-delete-char-backward-maybe
+    "Delete char backward when text is not the prefix helm is completing against.
+First call warn user about deleting prefix completion.
+Second call delete backward char in current-buffer and quit helm completion,
+letting user starting a new completion with a new prefix."
+  '(helm-mode-delete-char-backward-1 helm-mode-delete-char-backward-2) 1)
+
 (defun helm--completion-in-region (start end collection &optional predicate)
   "Helm replacement of `completion--in-region'.
 Can be used as value for `completion-in-region-function'."
@@ -1337,13 +1390,15 @@ Can be used as value for `completion-in-region-function'."
                           :initial-input
                           (cond ((and file-comp-p
                                       (not (string-match "/\\'" input)))
-                                 (concat (helm-basename input)
+                                 (concat (helm-mode--completion-in-region-initial-input
+                                          (helm-basename input))
                                          init-space-suffix))
                                 ((string-match "/\\'" input) nil)
                                 ((or (null require-match)
                                      (stringp require-match))
-                                 input)
-                                (t (concat input init-space-suffix)))
+                                 (helm-mode--completion-in-region-initial-input input))
+                                (t (concat (helm-mode--completion-in-region-initial-input input)
+                                           init-space-suffix)))
                           :buffer buf-name
                           :fc-transformer (append '(helm-cr-default-transformer)
                                                   (unless (or helm-completion-in-region-fuzzy-match
